@@ -34,8 +34,10 @@
 import sys
 
 from logilab.common import testlib
+from logilab.astng.node_classes import unpack_infer
+from logilab.astng.bases import YES, InferenceContext
 from logilab.astng.exceptions import ASTNGBuildingException, NotFoundError
-from logilab.astng import builder, nodes
+from logilab.astng import BUILTINS_MODULE, builder, nodes
 from logilab.astng.as_string import as_string
 
 from data import module as test_module
@@ -134,7 +136,7 @@ elif func():
 else:
     raise
     """
-        
+
     def test_if_elif_else_node(self):
         """test transformation for If node"""
         self.assertEqual(len(self.astng.body), 4)
@@ -231,25 +233,25 @@ MODULE2 = abuilder.file_build(join(DATA, 'module2.py'), 'data.module2')
 
 
 class ImportNodeTC(testlib.TestCase):
-    
+
     def test_import_self_resolve(self):
         myos = MODULE2.igetattr('myos').next()
         self.failUnless(isinstance(myos, nodes.Module), myos)
         self.failUnlessEqual(myos.name, 'os')
         self.failUnlessEqual(myos.qname(), 'os')
-        self.failUnlessEqual(myos.pytype(), '__builtin__.module')
+        self.failUnlessEqual(myos.pytype(), '%s.module' % BUILTINS_MODULE)
 
     def test_from_self_resolve(self):
         spawn = MODULE.igetattr('spawn').next()
         self.failUnless(isinstance(spawn, nodes.Class), spawn)
         self.failUnlessEqual(spawn.root().name, 'logilab.common.shellutils')
         self.failUnlessEqual(spawn.qname(), 'logilab.common.shellutils.Execute')
-        self.failUnlessEqual(spawn.pytype(), '__builtin__.classobj')
+        self.failUnlessEqual(spawn.pytype(), '%s.classobj' % BUILTINS_MODULE)
         abspath = MODULE2.igetattr('abspath').next()
         self.failUnless(isinstance(abspath, nodes.Function), abspath)
         self.failUnlessEqual(abspath.root().name, 'os.path')
         self.failUnlessEqual(abspath.qname(), 'os.path.abspath')
-        self.failUnlessEqual(abspath.pytype(), '__builtin__.function')
+        self.failUnlessEqual(abspath.pytype(), '%s.function' % BUILTINS_MODULE)
 
     def test_real_name(self):
         from_ = MODULE['spawn']
@@ -278,6 +280,43 @@ from ..cave import wine\n\n"""
         ast = abuilder.string_build(code)
         self.assertMultiLineEqual(ast.as_string(), code)
 
+    def test_bad_import_inference(self):
+        # Explication of bug
+        '''When we import PickleError from nonexistent, a call to the infer
+        method of this From node will be made by unpack_infer.
+        inference.infer_from will try to import this module, which will fail and
+        raise a InferenceException (by mixins.do_import_module). The infer_name
+        will catch this exception and yield and YES instead.
+        '''
+
+        code = '''try:
+    from pickle import PickleError
+except ImportError:
+    from nonexistent import PickleError
+
+try:
+    pass
+except PickleError:
+    pass
+        '''
+
+        astng = abuilder.string_build(code)
+        from_node = astng.body[1].handlers[0].body[0]
+        handler_type = astng.body[1].handlers[0].type
+
+        excs = list(unpack_infer(handler_type))
+
+    def test_absolute_import(self):
+        astng = abuilder.file_build(self.datapath('absimport.py'))
+        ctx = InferenceContext()
+        ctx.lookupname = 'message'
+        # will fail if absolute import failed
+        astng['message'].infer(ctx).next()
+        ctx.lookupname = 'email'
+        m = astng['email'].infer(ctx).next()
+        self.assertFalse(m.file.startswith(self.datapath('email.py')))
+
+
 class CmpNodeTC(testlib.TestCase):
     def test_as_string(self):
         ast = abuilder.string_build("a == 2").body[0]
@@ -285,7 +324,7 @@ class CmpNodeTC(testlib.TestCase):
 
 
 class ConstNodeTC(testlib.TestCase):
-    
+
     def _test(self, value):
         node = nodes.const_factory(value)
         self.assertIsInstance(node._proxied, nodes.Class)
@@ -293,25 +332,25 @@ class ConstNodeTC(testlib.TestCase):
         self.assertIs(node.value, value)
         self.failUnless(node._proxied.parent)
         self.assertEqual(node._proxied.root().name, value.__class__.__module__)
-        
+
     def test_none(self):
         self._test(None)
-        
+
     def test_bool(self):
         self._test(True)
-        
+
     def test_int(self):
         self._test(1)
-        
+
     def test_float(self):
         self._test(1.0)
-        
+
     def test_complex(self):
         self._test(1.0j)
-        
+
     def test_str(self):
         self._test('a')
-        
+
     def test_unicode(self):
         self._test(u'a')
 
@@ -335,7 +374,7 @@ del True
             del_true = ast.body[2].targets[0]
             self.assertIsInstance(del_true, nodes.DelName)
             self.assertEqual(del_true.name, "True")
-            
+
 
 class ArgumentsNodeTC(testlib.TestCase):
     def test_linenumbering(self):
@@ -387,6 +426,6 @@ class EllipsisNodeTC(testlib.TestCase):
     def test(self):
         ast = abuilder.string_build('a[...]').body[0]
         self.assertEqual(ast.as_string(), 'a[...]')
-        
+
 if __name__ == '__main__':
     testlib.unittest_main()
